package org.zipcoder.store;

import java.util.*;

public class MyHashMap implements MyMap {
    // final field that doesn't change for any object
    private static final int BUCKET_SIZE = 15;

    // instance field that can only be set once. It is different for every Object
    private final List<Entry>[] entries;
    private int entryListSize;

    public MyHashMap() {
        entries = new List[BUCKET_SIZE];

        for (int i = 0; i < BUCKET_SIZE; i++) {
            entries[i] = new ArrayList<>();
        }

        this.entryListSize = 0;
    }

    @Override
    public int size() {
        return this.entryListSize;
    }

    @Override
    public boolean isEmptry() {
        if (this.entryListSize == 0) {
            return true;
        }
        return false;
    }

    @Override
    public Cart get(User key) {
        List<Entry> bucket = entries[bucketIndex(key)];
        for (Entry entry : bucket) {
            if (entry.getKey().equals(key)) {
                return entry.getValue();
            }
        }
        return null;
    }

    @Override
    public void put(User key, Cart value) {
        List<Entry> bucket = entries[bucketIndex(key)];


        if (get(key) != null) {
            for (Entry entry : bucket) {
                if (entry.getKey().equals(key)) {
                    entry.setValue(value);
                }
            }
        } else {
            Entry entry = new Entry(key, value);
            bucket.add(entry);
            setEntryListSize(getEntryListSize() + 1);
        }
    }

    @Override
    public List<User> getKeys() {
        List<User> userKeys = new ArrayList<>();

        for (List<Entry> entryArray : entries) {
            for (Entry entry : entryArray) {
                userKeys.add(entry.getKey());
            }
        }

        return userKeys;
    }

    @Override
    public List<Cart> getValues() {
        List<Cart> userValues = new ArrayList<>();

        for (List<Entry> entryArray : entries) {
            for (Entry entry : entryArray) {
                userValues.add(entry.getValue());
            }
        }

        return userValues;
    }

    @Override
    public Cart remove(User key) {
        for (List<Entry> entryArray : entries) {
            for (Entry entry : entryArray) {
                if (entry.getKey().equals(key)) {
                    entryArray.remove(entry);
                    return entry.getValue();
                }
            }
        }
        return null;
    }

    private int bucketIndex(User key) {
        return Math.abs(key.hashCode()) % entries.length;
    }

    public int getEntryListSize() {
        return entryListSize;
    }

    public void setEntryListSize(int entryListSize) {
        this.entryListSize = entryListSize;
    }

    public List<Entry>[] getEntries() {
        return entries;
    }
}
